@@ -40,7 +40,7 @@ import uuid
 import os
 import decorator
 
-from errormator_client import __version__, __protocol_version__ 
+from errormator_client import __version__, __protocol_version__
 from errormator_client.ext_json import json
 from errormator_client.utils import asbool, aslist
 from errormator_client.timing import local_timing, get_local_storage
@@ -66,7 +66,7 @@ log = logging.getLogger(__name__)
 
 class Client(object):
     __version__ = __version__
-    __protocol_version__ = __protocol_version__ 
+    __protocol_version__ = __protocol_version__
 
     def __init__(self, config=None, register_timing=True):
         """
@@ -106,6 +106,8 @@ class Client(object):
                             config.get('errormator.logging_on_error', False))
         self.config['report_404'] = asbool(config.get('errormator.report_404',
                                                       False))
+        self.config['report_local_vars'] = asbool(
+                            config.get('errormator.report_local_vars', False))
         self.config['report_errors'] = asbool(
                                 config.get('errormator.report_errors', True))
         self.config['buffer_flush_interval'] = int(
@@ -114,7 +116,8 @@ class Client(object):
                                                       False))
         self.config['request_keys_blacklist'] = ['password', 'passwd', 'pwd',
                                                  'auth_tkt', 'secret', 'csrf',
-                                                 'session']
+                                                 'session', 'pass', 'config',
+                                                 'settings', 'environ']
         user_blacklist = aslist(config.get('errormator.request_keys_blacklist',
                             config.get('errormator.bad_request_keys')), ',')
         self.config['request_keys_blacklist'].extend(
@@ -281,17 +284,38 @@ class Client(object):
             log.error(message)
 
     def data_filter(self, structure, section=None):
+        def filter_dict(input, dict_method):
+            for k in dict_method():
+                for bad_key in self.config['request_keys_blacklist']:
+                    if (bad_key in k.lower()):
+                        input[k] = u'***'
+
         if section in ['error_report', 'slow_report']:
             keys_to_check = (
                     structure['report_details'][0]['request'].get('COOKIES'),
-                    structure['report_details'][0]['request'].get('POST')
+                    structure['report_details'][0]['request'].get('POST'),
                     )
-
         for source in filter(None, keys_to_check):
-            for k in source.iterkeys():
-                for bad_key in self.config['request_keys_blacklist']:
-                    if (bad_key in k.lower()):
-                        source[k] = u'***'
+            if hasattr(source, 'iterkeys'):
+                filter_dict(source, source.iterkeys)
+            elif hasattr(source, 'keys'):
+                filter_dict(source, source.keys)
+        # try to filter local frame vars, to prevent people leaking as much
+        # data as possible when enabling frameinfo
+        frameinfo = structure['report_details'][0].get('frameinfo')
+        if frameinfo:
+            for f in frameinfo:
+                for source in f.get('vars', []):
+                    # filter dict likes
+                    if hasattr(source[1], 'iterkeys'):
+                        filter_dict(source[1], source[1].iterkeys)
+                    elif hasattr(source, 'keys'):
+                        filter_dict(source[1], source[1].keys)
+                    # filter flat values
+                    else:
+                        for bad_key in self.config['request_keys_blacklist']:
+                            if (bad_key in source[0].lower()):
+                                source[1] = u'***'
         return structure
 
     def py_report(self, environ, traceback=None, message=None, http_status=200,
@@ -322,7 +346,8 @@ class Client(object):
             records = self.log_handler.get_records()
             self.log_handler.clear_records()
 
-        if self.config['logging_on_error'] and traceback is None:
+        if not environ.get('errormator.force_logs') and \
+            (self.config['logging_on_error'] and traceback is None):
             return False
 
         for record in records:
@@ -403,7 +428,8 @@ class Client(object):
         for key, value in req.environ.items():
             if key.startswith('errormator.') \
             and key not in ('errormator.client', 'errormator.force_send',
-                            'errormator.log', 'errormator.report'):
+                            'errormator.log', 'errormator.report',
+                            'errormator.force_logs'):
                 errormator_info[key[11:]] = unicode(value)
             else:
                 if traceback and (key.startswith('HTTP') or \
@@ -463,18 +489,21 @@ class Client(object):
                                                             include_params)
         report_data = {'client': 'Python', 'report_details': []}
         report_data['error_type'] = 'Unknown'
+        detail_entry = {}
         if traceback:
             exception_text = traceback.exception
             traceback_text = traceback.plaintext
             report_data['error_type'] = exception_text
-            report_data['traceback'] = traceback_text
+            local_vars = (self.config['report_local_vars'] or
+                          environ.get('errormator.report_local_vars'))
+            detail_entry['frameinfo'] = traceback.frameinfo(include_vars=local_vars)
+
         report_data['http_status'] = 500 if traceback else http_status
         if http_status == 404:
             report_data['error_type'] = '404 Not Found'
         report_data['priority'] = 5
         report_data['server'] = (server or
                     environ.get('SERVER_NAME', 'unknown server'))
-        detail_entry = {}
         detail_entry['request'] = parsed_environ
         # fill in all other required info
         detail_entry['ip'] = parsed_environ.get('REMOTE_ADDR', u'')
